@@ -16,15 +16,15 @@
 #' is_zero(unknown)
 #' is_zero(pos + neg)
 #'
-#' is_positive(pos + zero)
-#' is_positive(pos * neg)
-#' is_positive(pos - neg)
-#' is_positive(unknown)
+#' is_nonneg(pos + zero)
+#' is_nonneg(pos * neg)
+#' is_nonneg(pos - neg)
+#' is_nonneg(unknown)
 #'
-#' is_negative(-pos)
-#' is_negative(pos + neg)
-#' is_negative(neg * zero)
-#' is_negative(neg - pos)
+#' is_nonpos(-pos)
+#' is_nonpos(pos + neg)
+#' is_nonpos(neg * zero)
+#' is_nonpos(neg - pos)
 #' @name sign-methods
 NULL
 
@@ -34,11 +34,33 @@ setGeneric("is_zero", function(object) { standardGeneric("is_zero") })
 
 #' @rdname sign-methods
 #' @export
-setGeneric("is_positive", function(object) { standardGeneric("is_positive") })
+setGeneric("is_nonneg", function(object) { standardGeneric("is_nonneg") })
 
 #' @rdname sign-methods
 #' @export
-setGeneric("is_negative", function(object) { standardGeneric("is_negative") })
+setGeneric("is_nonpos", function(object) { standardGeneric("is_nonpos") })
+
+#'
+#' Complex Properties
+#'
+#' Determine if an expression is real, imaginary, or complex.
+#'
+#' @param object An \linkS4class{Expression} object.
+#' @return A logical value.
+#' @name complex-methods
+NULL
+
+#' @rdname complex-methods
+#' @export
+setGeneric("is_real", function(object) { standardGeneric("is_real") })
+
+#' @rdname complex-methods
+#' @export
+setGeneric("is_imag", function(object) { standardGeneric("is_imag") })
+
+#' @rdname complex-methods
+#' @export
+setGeneric("is_complex", function(object) { standardGeneric("is_complex") })
 
 #'
 #' Curvature of Expression
@@ -64,7 +86,7 @@ setGeneric("curvature", function(object) { standardGeneric("curvature") })
 #'
 #' Curvature Properties
 #'
-#' Determine if an expression is constant, affine, convex, concave, quadratic, or piecewise linear (pwl).
+#' Determine if an expression is constant, affine, convex, concave, quadratic, piecewise linear (pwl), or quadratic/piecewise affine (qpwa).
 #'
 #' @param object An \linkS4class{Expression} object.
 #' @return A logical value.
@@ -122,6 +144,47 @@ setGeneric("is_quadratic", function(object) { standardGeneric("is_quadratic") })
 #' @export
 setGeneric("is_pwl", function(object) { standardGeneric("is_pwl") })
 
+#' @rdname curvature-methods
+#' @export
+setGeneric("is_qpwa", function(object) { standardGeneric("is_qpwa") })
+
+#'
+#' Log-Log Curvature of Expression
+#'
+#' The log-log curvature of an expression.
+#'
+#' @param object An \linkS4class{Expression} object.
+#' @return A string indicating the log-log curvature of the expression, either "LOG_LOG_CONSTANT", "LOG_LOG_AFFINE", "LOG_LOG_CONVEX, "LOG_LOG_CONCAVE", or "UNKNOWN".
+#' @docType methods
+#' @rdname log_log_curvature
+setGeneric("log_log_curvature", function(object) { standardGeneric("log_log_curvature") })
+
+#'
+#' Log-Log Curvature Properties
+#'
+#' Determine if an expression is log-log constant, log-log affine, log-log convex, or log-log concave.
+#'
+#' @param object An \linkS4class{Expression} object.
+#' @return A logical value.
+#' @name log_log_curvature-methods
+NULL
+
+#' @rdname log_log_curvature-methods
+#' @export
+setGeneric("is_log_log_constant", function(object) { standardGeneric("is_log_log_constant") })
+
+#' @rdname log_log_curvature-methods
+#' @export
+setGeneric("is_log_log_affine", function(object) { standardGeneric("is_log_log_affine") })
+
+#' @rdname log_log_curvature-methods
+#' @export
+setGeneric("is_log_log_convex", function(object) { standardGeneric("is_log_log_convex") })
+
+#' @rdname log_log_curvature-methods
+#' @export
+setGeneric("is_log_log_concave", function(object) { standardGeneric("is_log_log_concave") })
+
 #'
 #' DCP Compliance
 #'
@@ -139,6 +202,24 @@ setGeneric("is_pwl", function(object) { standardGeneric("is_pwl") })
 #' @export
 setGeneric("is_dcp", function(object) { standardGeneric("is_dcp") })
 
+#'
+#' DGP Compliance
+#'
+#' Determine if a problem or expression complies with the disciplined geometric programming rules.
+#'
+#' @param object A \linkS4class{Problem} or \linkS4class{Expression} object.
+#' @return A logical value indicating whether the problem or expression is DCP compliant, i.e. no unknown curvatures.
+#' @examples
+#' x <- Variable(pos = TRUE)
+#' y <- Variable(pos = TRUE)
+#' prob <- Problem(Minimize(x*y), list(x >= 5, y >= 5))
+#' is_dgp(prob)
+#' solve(prob, gp = TRUE)
+#' @docType methods
+#' @rdname is_dgp
+#' @export
+setGeneric("is_dgp", function(object) { standardGeneric("is_dgp") })
+
 #'
 #' Size of Expression
 #'
@@ -200,6 +281,32 @@ setGeneric("is_vector", function(object) { standardGeneric("is_vector") })
 #' @export
 setGeneric("is_matrix", function(object) { standardGeneric("is_matrix") })
 
+#'
+#' Matrix Properties
+#'
+#' Determine if an expression is positive semidefinite, negative semidefinite, hermitian, and/or symmetric.
+#'
+#' @param object An \linkS4class{Expression} object.
+#' @return A logical value.
+#' @name matrix_prop-methods
+NULL
+
+#' @rdname matrix_prop-methods
+#' @export
+setGeneric("is_psd", function(object) { standardGeneric("is_psd") })
+
+#' @rdname matrix_prop-methods
+#' @export
+setGeneric("is_nsd", function(object) { standardGeneric("is_nsd") })
+
+#' @rdname matrix_prop-methods
+#' @export
+setGeneric("is_hermitian", function(object) { standardGeneric("is_hermitian") })
+
+#' @rdname matrix_prop-methods
+#' @export
+setGeneric("is_symmetric", function(object) { standardGeneric("is_symmetric") })
+
 # The value of the objective given the solver primal value.
 setGeneric("primal_to_result", function(object, result) { standardGeneric("primal_to_result") })
 
@@ -231,6 +338,19 @@ setGeneric("value<-", function(object, value) { standardGeneric("value<-") })
 # Internal method for saving the value of an expression
 setGeneric("save_value", function(object, value) { standardGeneric("save_value") })
 
+#'
+#' Is Constraint Violated?
+#'
+#' Checks whether the constraint violation is less than a tolerance.
+#'
+#' @param object A \linkS4class{Constraint} object.
+#' @param tolerance A numeric scalar representing the absolute tolerance to impose on the violation.
+#' @return A logical value indicating whether the violation is less than the \code{tolerance}. Raises an error if the residual is \code{NA}.
+#' @docType methods
+#' @rdname constr_value
+#' @export
+setGeneric("constr_value", function(object, tolerance = 1e-8) { standardGeneric("constr_value") })
+
 #'
 #' Get Expression Data
 #'
@@ -248,7 +368,7 @@ setGeneric("get_data", function(object) { standardGeneric("get_data") })
 #'
 #' The string representation of a variable, parameter, or expression.
 #'
-#' @param object A \linkS4class{Variable}, \linkS4class{Parameter}, or \linkS4class{Expression} object.
+#' @param x A \linkS4class{Variable}, \linkS4class{Parameter}, or \linkS4class{Expression} object.
 #' @return For \linkS4class{Variable} or \linkS4class{Parameter} objects, the value in the name slot. For \linkS4class{Expression} objects, a string indicating the nested atoms and their respective arguments.
 #' @docType methods
 #' @rdname name
@@ -259,16 +379,17 @@ setGeneric("get_data", function(object) { standardGeneric("get_data") })
 #' name(x)
 #' name(y)
 #' @export
-setGeneric("name", function(object) { standardGeneric("name") })
+setGeneric("name", function(x) { standardGeneric("name") })
 
 #'
-#' Parts of an Expression
+#' Parts of an Expression Leaf
 #'
-#' List the variables, parameters, or constants in a canonical expression.
+#' List the variables, parameters, constants, or atoms in a canonical expression.
 #'
-#' @param object A \linkS4class{Canonical} expression.
-#' @return A list of \linkS4class{Variable}, \linkS4class{Parameter}, or \linkS4class{Constant} objects.
+#' @param object A \linkS4class{Leaf} object.
+#' @return A list of \linkS4class{Variable}, \linkS4class{Parameter}, \linkS4class{Constant}, or \linkS4class{Atom} objects.
 #' @examples
+#' set.seed(67)
 #' m <- 50
 #' n <- 10
 #' beta <- Variable(n)
@@ -296,6 +417,28 @@ setGeneric("parameters", function(object) { standardGeneric("parameters") })
 #' @export
 setGeneric("constants", function(object) { standardGeneric("constants") })
 
+#' @rdname expression-parts
+#' @export
+setGeneric("atoms", function(object) { standardGeneric("atoms") })
+
+#'
+#' Attributes of an Expression Leaf
+#'
+#' Determine if an expression is positive or negative.
+#'
+#' @param object A \linkS4class{Leaf} object.
+#' @return A logical value.
+#' @name leaf-attr
+NULL
+
+#' @rdname leaf-attr
+#' @export
+setGeneric("is_pos", function(object) { standardGeneric("is_pos") })
+
+#' @rdname leaf-attr
+#' @export
+setGeneric("is_neg", function(object) { standardGeneric("is_neg") })
+
 #'
 #' Sub/Super-Gradient
 #'
@@ -359,6 +502,32 @@ setGeneric("grad", function(object) { standardGeneric("grad") })
 #' @export
 setGeneric("domain", function(object) { standardGeneric("domain") })
 
+#'
+#' Project Value
+#'
+#' Project a value onto the attribute set of a \linkS4class{Leaf}.
+#' A sensible idiom is \code{value(leaf) = project(leaf, val)}.
+#'
+#' @param object A \linkS4class{Leaf} object.
+#' @param value The assigned value.
+#' @return The value rounded to the attribute type.
+#' @name project-methods
+NULL
+
+#'
+#' Project a value onto the attribute set of a \linkS4class{Leaf}.
+#'
+#' @rdname project-methods
+#' @export
+setGeneric("project", function(object, value) { standardGeneric("project") })
+
+#'
+#' Projects and assigns a value onto the attribute set of a \linkS4class{Leaf}.
+#'
+#' @rdname project-methods
+#' @export
+setGeneric("project_and_assign", function(object, value) { standardGeneric("project_and_assign") })
+
 #'
 #' Validate Value
 #'
@@ -435,14 +604,14 @@ setGeneric("%>>%", function(e1, e2) { standardGeneric("%>>%") })
 setGeneric("%<<%", function(e1, e2) { standardGeneric("%<<%") })
 
 #'
-#' Atom Size
+#' Atom Dimensions
 #'
-#' Determine the size of an atom based on its arguments.
+#' Determine the dimensions of an atom based on its arguments.
 #'
 #' @param object A \linkS4class{Atom} object.
-#' @return A numeric vector \code{c(row, col)} indicating the size of the atom.
-#' @rdname size_from_args
-setGeneric("size_from_args", function(object) { standardGeneric("size_from_args") })
+#' @return A numeric vector \code{c(row, col)} indicating the dimensions of the atom.
+#' @rdname dim_from_args
+setGeneric("dim_from_args", function(object) { standardGeneric("dim_from_args") })
 
 #'
 #' Atom Sign
@@ -511,6 +680,28 @@ setGeneric("is_atom_concave", function(object) { standardGeneric("is_atom_concav
 #' @export
 setGeneric("is_atom_affine", function(object) { standardGeneric("is_atom_affine") })
 
+#'
+#' Log-Log Curvature of an Atom
+#'
+#' Determine if an atom is log-log convex, concave, or affine.
+#'
+#' @param object A \linkS4class{Atom} object.
+#' @return A logical value.
+#' @name log_log_curvature-atom
+NULL
+
+#' @rdname log_log_curvature-atom
+#' @export
+setGeneric("is_atom_log_log_convex", function(object) { standardGeneric("is_atom_log_log_convex") })
+
+#' @rdname log_log_curvature-atom
+#' @export
+setGeneric("is_atom_log_log_concave", function(object) { standardGeneric("is_atom_log_log_concave") })
+
+#' @rdname log_log_curvature-atom
+#' @export
+setGeneric("is_atom_log_log_affine", function(object) { standardGeneric("is_atom_log_log_affine") })
+
 #'
 #' Curvature of Composition
 #'
@@ -543,12 +734,12 @@ setGeneric("is_decr", function(object, idx) { standardGeneric("is_decr") })
 #'
 #' @param object An \linkS4class{Expression} object.
 #' @param arg_objs A list of linear expressions for each argument.
-#' @param size A vector with two elements representing the size of the resulting expression.
+#' @param dim A vector representing the dimensions of the resulting expression.
 #' @param data A list of additional data required by the atom.
 #' @return A list of \code{list(LinOp for objective, list of constraints)}, where LinOp is a list representing the linear operator.
 #' @docType methods
 #' @rdname graph_implementation
-setGeneric("graph_implementation", function(object, arg_objs, size, data) { standardGeneric("graph_implementation") })
+setGeneric("graph_implementation", function(object, arg_objs, dim, data) { standardGeneric("graph_implementation") })
 
 #'
 #' Identification Number
@@ -581,15 +772,17 @@ setGeneric("id", function(object) { standardGeneric("id") })
 NULL
 
 #' @rdname residual-methods
+#' @export
 setGeneric("residual", function(object) { standardGeneric("residual") })
 
 #' @rdname residual-methods
+#' @export
 setGeneric("violation", function(object) { standardGeneric("violation") })
 
 #'
 #' Second-Order Cone Methods
 #'
-#' The number of elementwise cones or the size of a single cone in a second-order cone constraint.
+#' The number of elementwise cones or a list of the sizes of the elementwise cones.
 #'
 #' @param object An \linkS4class{SOCAxis} object.
 #' @return The number of cones, or the size of a cone.
@@ -601,18 +794,29 @@ NULL
 setGeneric("num_cones", function(object) { standardGeneric("num_cones") })
 
 #' @rdname cone-methods
-setGeneric("cone_size", function(object) { standardGeneric("cone_size") })
+setGeneric("cone_sizes", function(object) { standardGeneric("cone_sizes") })
 
-#
-# Dual Value
-#
-# The value of the dual variable in a constraint.
-#
-# @param object A \linkS4class{Constraint} object.
-# @return The numeric value of the dual variable. Defaults to \code{NA} if unknown.
-# @rdname dual_value
+#'
+#' Get and Set Dual Value
+#'
+#' Get and set the value of the dual variable in a constraint.
+#'
+#' @param object A \linkS4class{Constraint} object.
+#' @param value A numeric scalar, vector, or matrix to assign to the object.
+#' @name dual_value-methods
+NULL
+
+#'
+#' Get and set the value of the dual variable in a constraint.
+#'
+#' @rdname dual_value-methods
+#' @export
 setGeneric("dual_value", function(object) { standardGeneric("dual_value") })
 
+#' @rdname dual_value-methods
+#' @export
+setGeneric("dual_value<-", function(object, value) { standardGeneric("dual_value<-") })
+
 #'
 #' Format Constraints
 #'
@@ -693,23 +897,24 @@ setGeneric("solver_stats<-", function(object, value) { standardGeneric("solver_s
 #'
 #' @param object A \linkS4class{Problem} object.
 #' @param solver A string indicating the solver that the problem data is for. Call \code{installed_solvers()} to see all available.
-#' @return A list of arguments for the solver.
+#' @param gp (Optional) A logical value indicating whether the problem is a geometric program.
+#' @return A list containing the data for the solver, the solving chain for the problem, and the inverse data needed to invert the solution.
 #' @examples
 #' a <- Variable(name = "a")
-#' data <- get_problem_data(Problem(Maximize(exp(a) + 2)), "SCS")
+#' data <- get_problem_data(Problem(Minimize(exp(a) + 2)), "SCS")[[1]]
 #' data[["dims"]]
 #' data[["c"]]
 #' data[["A"]]
 #'
 #' x <- Variable(2, name = "x")
-#' data <- get_problem_data(Problem(Minimize(p_norm(x) + 3)), "ECOS")
+#' data <- get_problem_data(Problem(Minimize(p_norm(x) + 3)), "ECOS")[[1]]
 #' data[["dims"]]
 #' data[["c"]]
 #' data[["A"]]
 #' data[["G"]]
 #' @rdname get_problem_data
 #' @export
-setGeneric("get_problem_data", function(object, solver) { standardGeneric("get_problem_data") })
+setGeneric("get_problem_data", function(object, solver, gp) { standardGeneric("get_problem_data") })
 
 #'
 #' Solve a DCP Problem
@@ -721,7 +926,12 @@ setGeneric("get_problem_data", function(object, solver) { standardGeneric("get_p
 #' @param ignore_dcp (Optional) A logical value indicating whether to override the DCP check for a problem.
 #' @param warm_start (Optional) A logical value indicating whether the previous solver result should be used to warm start.
 #' @param verbose (Optional) A logical value indicating whether to print additional solver output.
+#' @param feastol The feasible tolerance on the primal and dual residual.
+#' @param reltol The relative tolerance on the duality gap.
+#' @param abstol The absolute tolerance on the duality gap.
+#' @param num_iter The maximum number of iterations.
 #' @param parallel (Optional) A logical value indicating whether to solve in parallel if the problem is separable.
+#' @param gp (Optional) A logical value indicating whether the problem is a geometric program. Defaults to \code{FALSE}.
 #' @param ... Additional options that will be passed to the specific solver. In general, these options will override any default settings imposed by CVXR.
 #' @return A list containing the solution to the problem:
 #' \describe{
@@ -746,7 +956,8 @@ setGeneric("get_problem_data", function(object, solver) { standardGeneric("get_p
 #' @aliases psolve solve
 #' @rdname psolve
 #' @export
-setGeneric("psolve", function(object, solver, ignore_dcp = FALSE, warm_start = FALSE, verbose = FALSE, parallel = FALSE, ...) { standardGeneric("psolve") })
+setGeneric("psolve", function(object, solver = NA, ignore_dcp = FALSE, warm_start = FALSE, verbose = FALSE, parallel = FALSE, 
+                              gp = FALSE, feastol = NULL, reltol = NULL, abstol = NULL, num_iter = NULL, ...) { standardGeneric("psolve") })
 
 #'
 #' Is Problem a QP?
@@ -760,14 +971,27 @@ setGeneric("psolve", function(object, solver, ignore_dcp = FALSE, warm_start = F
 #' @export
 setGeneric("is_qp", function(object) { standardGeneric("is_qp") })
 
+#'
+#' Is Problem Mixed Integer?
+#'
+#' Determine if a problem is a mixed-integer program.
+#'
+#' @param object A \linkS4class{Problem} object.
+#' @return A logical value indicating whether the problem is a mixed-integer program
+#' @docType methods
+#' @rdname is_mixed_integer
+#' @export
+setGeneric("is_mixed_integer", function(object) { standardGeneric("is_mixed_integer") })
+
 #'
 #' Parse output from a solver and updates problem state
 #'
 #' Updates problem status, problem value, and primal and dual variable values
 #'
 #' @param object A \linkS4class{Problem} object.
-#' @param solver A character string specifying the solver such as "ECOS", "SCS" etc.
-#' @param results_dict the solver output
+#' @param solution A \linkS4class{Solution} object.
+#' @param chain The corresponding solving \linkS4class{Chain}.
+#' @param inverse_data A \linkS4class{InverseData} object or list containing data necessary for the inversion.
 #' @return A list containing the solution to the problem:
 #' \describe{
 #'    \item{\code{status}}{The status of the solution. Can be "optimal", "optimal_inaccurate", "infeasible", "infeasible_inaccurate", "unbounded", "unbounded_inaccurate", or "solver_error".}
@@ -800,7 +1024,7 @@ setGeneric("is_qp", function(object) { standardGeneric("is_qp") })
 #' res1 <- unpack_results(prob1, "ECOS", ecos_output)
 #' # Without DCP validation (so be sure of your math), above is equivalent to:
 #' # res1 <- solve(prob1, solver = "ECOS")
-#' X <- Semidef(2)
+#' X <- Variable(2,2, PSD = TRUE)
 #' Fmat <- rbind(c(1,0), c(0,-1))
 #' obj <- Minimize(sum_squares(X - Fmat))
 #' prob2 <- Problem(obj)
@@ -836,7 +1060,7 @@ setGeneric("get_nonlin_constr", function(object) { standardGeneric("get_nonlin_c
 #'
 #' Import the R library that interfaces with the specified solver.
 #'
-#' @param solver A \linkS4class{Solver} object.
+#' @param solver A \linkS4class{ReductionSolver} object.
 #' @examples
 #' import_solver(ECOS())
 #' import_solver(SCS())
@@ -846,156 +1070,129 @@ setGeneric("get_nonlin_constr", function(object) { standardGeneric("get_nonlin_c
 setGeneric("import_solver", function(solver) { standardGeneric("import_solver") })
 setGeneric("is_installed", function(solver) { standardGeneric("is_installed") })
 
-#
-# Choose MOSEK Solution
-#
-# Chooses between the basic and interior point solution from MOSEK. Solutions are ranked optimal > near_optimal > anything else > None.
-# As long as interior solution is not worse, take it (for backward compatibility).
-#
-# @param solver A \linkS4class{MOSEK} object.
-# @param results_dict A list of the results returned by the solver.
-# @return A list containing the preferred solution (\code{solist}) and status of the preferred solution (\code{solsta}).
-# @rdname choose_solution
-# setGeneric("choose_solution", function(solver, results_dict) { standardGeneric("choose_solution") })
-
-setGeneric("nonlin_constr", function(solver) { standardGeneric("nonlin_constr") })
-
-#'
-#' Validate Solver
-#'
-#' Raises an exception if the solver cannot solve the problem.
-#'
-#' @param solver A \linkS4class{Solver} object.
-#' @param constraints A list of canonicalized constraints
-#' @docType methods
-#' @rdname validate_solver
-setGeneric("validate_solver", function(solver, constraints) { standardGeneric("validate_solver") })
-
-#
-# Validate Cache
-#
-# Clears the cache if the objective or constraints changed.
-#
-# @param solver A \linkS4class{Solver} object.
-# @param objective A list representing the canonicalized objective.
-# @param constraints A list of canonicalized constraints.
-# @param cached_data A list mapping solver name to cached problem data.
-# @return The updated \code{cached_data}.
-setGeneric("validate_cache", function(solver, objective, constraints, cached_data) { standardGeneric("validate_cache") })
-
-#
-# Get Symbolic Data
-#
-# Returns the symbolic data for the problem.
-#
-# @param solver A \linkS4class{Solver} object.
-# @param objective A list representing the canonicalized objective.
-# @param constraints A list of canonicalized constraints.
-# @param cached_data A list mapping solver name to cached problem data.
-# @return A \linkS4class{SymData} object holding the symbolic data for the problem.
-setGeneric("get_sym_data", function(solver, objective, constraints, cached_data) { standardGeneric("get_sym_data") })
-
-#
-# Get Matrix Data
-#
-# Returns the numeric data for the problem.
-#
-# @param solver A \linkS4class{Solver} object.
-# @param objective A list representing the canonicalized objective.
-# @param constraints A list of canonicalized constraints.
-# @param cached_data A list mapping solver name to cached problem data.
-# @return A \linkS4class{SymData} object holding the symbolic data for the problem.
-setGeneric("get_matrix_data", function(solver, objective, constraints, cached_data) { standardGeneric("get_matrix_data") })
-
-#
-# Solver: Get Problem Data
-#
-# Returns the argument for the call to the solver.
-#
-# @param solver A \linkS4class{Solver} object.
-# @param objective A list representing the canonicalized objective.
-# @param constraints A list of canonicalized constraints.
-# @param cached_data A list mapping solver name to cached problem data.
-# @return A list of the arguments needed for the solver.
-setGeneric("Solver.get_problem_data", function(solver, objective, constraints, cached_data) { standardGeneric("Solver.get_problem_data") })
-
-#
-# Extract Constraints
-#
-# Extracts the equality, inequality, and nonlinear constraints.
-#
-# @param solver A \linkS4class{Solver} object.
-# @param constr_map A list of canonicalized constraints.
-# @return A list of equality, inequality, and nonlinear constraints.
-setGeneric("split_constr", function(solver, constr_map) { standardGeneric("split_constr") })
-
-#'
-#' Call to Solver
-#'
-#' Returns the result of the call to the solver.
-#'
-#' @param solver A \linkS4class{Solver} object.
-#' @param objective A list representing the canonicalized objective.
-#' @param constraints A list of canonicalized constraints.
-#' @param cached_data A list mapping solver name to cached problem data.
-#' @param warm_start A logical value indicating whether the previous solver result should be used to warm start.
-#' @param verbose A logical value indicating whether to print solver output.
-#' @param ... Additional arguments to the solver.
-#' @return A list containing the status, optimal value, primal variable, and dual variables for the equality and inequality constraints.
-#' @docType methods
-#' @aliases Solver.solve
-#' @rdname Solver-solve
-setGeneric("Solver.solve", function(solver, objective, constraints, cached_data, warm_start, verbose, ...) { standardGeneric("Solver.solve") })
-
-#'
-#' Format Solver Results
-#'
-#' Converts the solver output into standard form.
-#'
-#' @param solver A \linkS4class{Solver} object.
-#' @param results_dict A list containing the solver output.
-#' @param data A list containing information about the problem.
-#' @param cached_data A list mapping solver name to cached problem data.
-#' @return A list containing the solver output in standard form.
-#' @docType methods
-#' @rdname format_results
-setGeneric("format_results", function(solver, results_dict, data, cached_data) { standardGeneric("format_results") })
-
 #'
 #' Solver Capabilities
 #'
-#' Determine if a solver is capable of solving a linear program (LP), second-order cone program (SOCP), semidefinite program (SDP), exponential cone program (EXP), or mixed-integer program (MIP).
+#' Determine if a solver is capable of solving a mixed-integer program (MIP).
 #'
-#' @param solver A \linkS4class{Solver} object.
+#' @param solver A \linkS4class{ReductionSolver} object.
 #' @return A logical value.
 #' @examples
-#' lp_capable(ECOS())
-#' socp_capable(ECOS())
-#' sdp_capable(ECOS())
-#' exp_capable(ECOS())
 #' mip_capable(ECOS())
-#' @name Solver-capable
-NULL
-
-#' @rdname Solver-capable
-#' @export
-setGeneric("lp_capable", function(solver) { standardGeneric("lp_capable") })
-
-#' @rdname Solver-capable
-#' @export
-setGeneric("socp_capable", function(solver) { standardGeneric("socp_capable") })
-
-#' @rdname Solver-capable
-#' @export
-setGeneric("sdp_capable", function(solver) { standardGeneric("sdp_capable") })
-
-#' @rdname Solver-capable
-#' @export
-setGeneric("exp_capable", function(solver) { standardGeneric("exp_capable") })
-
-#' @rdname Solver-capable
+#' @rdname mip_capable
+#' @docType methods
 #' @export
 setGeneric("mip_capable", function(solver) { standardGeneric("mip_capable") })
 
 # Map of solver status code to CVXR status.
 setGeneric("status_map", function(solver, status) { standardGeneric("status_map") })
+
+# Map of CBC MIP/LP status to CVXR status.
+setGeneric("status_map_mip", function(solver, status) { standardGeneric("status_map_mip") })
+setGeneric("status_map_lp", function(solver, status) { standardGeneric("status_map_lp") })
+
+#'
+#' Reduction Acceptance
+#'
+#' Determine whether the reduction accepts a problem.
+#'
+#' @param object A \linkS4class{Reduction} object.
+#' @param problem A \linkS4class{Problem} to check.
+#' @return A logical value indicating whether the reduction can be applied.
+#' @docType methods
+#' @rdname accepts
+setGeneric("accepts", function(object, problem) { standardGeneric("accepts") })
+
+#'
+#' Reduce a Problem
+#'
+#' Reduces the owned problem to an equivalent problem.
+#'
+#' @param object A \linkS4class{Reduction} object.
+#' @return An equivalent problem, encoded either as a \linkS4class{Problem} object or a list.
+#' @docType methods
+#' @rdname reduce
+setGeneric("reduce", function(object) { standardGeneric("reduce") })
+
+#'
+#' Retrieve Solution
+#'
+#' Retrieves a solution to the owned problem.
+#'
+#' @param object A \linkS4class{Reduction} object.
+#' @param solution A \linkS4class{Solution} object.
+#' @return A \linkS4class{Solution} to the problem emitted by \code{\link{reduce}}.
+#' @docType methods
+#' @rdname retrieve
+setGeneric("retrieve", function(object, solution) { standardGeneric("retrieve") })
+
+#'
+#' Perform Reduction
+#'
+#' Performs the reduction on a problem and returns an equivalent problem.
+#'
+#' @param object A \linkS4class{Reduction} object.
+#' @param problem A \linkS4class{Problem} on which the reduction will be performed.
+#' @return A list containing
+#' \itemize{
+#'    \item{"problem"}{A \linkS4class{Problem} or list representing the equivalent problem.}
+#'    \item{"inverse_data"}{A \linkS4class{InverseData} or list containing the data needed to invert this particular reduction.}
+#' }
+#' @docType methods
+#' @rdname perform
+setGeneric("perform", function(object, problem) { standardGeneric("perform") })
+
+#'
+#' Return Original Solution
+#'
+#' Returns a solution to the original problem given the inverse data.
+#'
+#' @param object A \linkS4class{Reduction} object.
+#' @param solution A \linkS4class{Solution} to a problem that generated \code{inverse_data}.
+#' @param inverse_data A \linkS4class{InverseData} object encoding the original problem.
+#' @return A \linkS4class{Solution} to the original problem.
+#' @docType methods
+#' @rdname invert
+setGeneric("invert", function(object, solution, inverse_data) { standardGeneric("invert") })
+##setGeneric("alt_invert", function(object, results, inverse_data) { standardGeneric("alt_invert") })
+
+## Version 1.0 edits
+setGeneric("square", function(x) { standardGeneric("square") })
+
+## Start of newly added generics
+setGeneric("expr", function(object) { standardGeneric("expr") })
+setGeneric("ndim", function(object) { standardGeneric("ndim") })
+setGeneric("flatten", function(object) { standardGeneric("flatten") })
+setGeneric("value_impl", function(object) { standardGeneric("value_impl") })
+setGeneric("var_id", function(object) { standardGeneric("var_id") })
+setGeneric("get_attr_str", function(object) { standardGeneric("get_attr_str") })
+setGeneric("get_var_offsets", function(object, variables) { standardGeneric("get_var_offsets") })
+setGeneric("allow_complex", function(object) { standardGeneric("allow_complex") })
+setGeneric("canonicalize_tree", function(object, expr) { standardGeneric("canonicalize_tree") })
+setGeneric("canonicalize_expr", function(object, expr, args) { standardGeneric("canonicalize_expr") })
+setGeneric("stuffed_objective", function(object, problem, extractor) { standardGeneric("stuffed_objective") })
+setGeneric("prepend", function(object, chain) { standardGeneric("prepend") })
+setGeneric("group_coeff_offset", function(object, problem, constraints, exp_cone_order) { standardGeneric("group_coeff_offset") })
+setGeneric("construct_intermediate_chain", function(problem, candidates, gp) { standardGeneric("construct_intermediate_chain") })
+setGeneric("solve_via_data", function(object, data, warm_start, verbose, feastol, reltol, abstol, num_iter, solver_opts, solver_cache) { standardGeneric("solve_via_data") })
+setGeneric("unpack_problem", function(object, solution) { standardGeneric("unpack_problem") })
+setGeneric("unpack_results", function(object, solution, chain, inverse_data) { standardGeneric("unpack_results") })
+setGeneric(".construct_dual_variables", function(object, args) { standardGeneric(".construct_dual_variables") })
+setGeneric("reduction_solve", function(object, problem, warm_start, verbose,  feastol, reltol, abstol, num_iter, solver_opts) { standardGeneric("reduction_solve") })
+setGeneric("reduction_solve_via_data", function(object, problem, data, warm_start, verbose, feastol, reltol, abstol, num_iter, solver_opts) { standardGeneric("reduction_solve_via_data") })
+setGeneric("reduction_format_constr", function(object, problem, constr, exp_cone_order) { standardGeneric("reduction_format_constr") })
+setGeneric("block_format", function(object, problem, constraints, exp_cone_order) { standardGeneric("block_format") })
+setGeneric("suitable", function(solver, problem) { standardGeneric("suitable") })
+setGeneric("ls_solve", function(object, objective, constraints, cached_data, warm_start, verbose, solver_opts) { standardGeneric("ls_solve") })
+setGeneric("supported_constraints", function(solver) { standardGeneric("supported_constraints") })
+setGeneric("requires_constr", function(solver) { standardGeneric("requires_constr") })
+setGeneric("get_coeffs", function(object, expr) { standardGeneric("get_coeffs") })
+setGeneric("constant", function(object, expr) { standardGeneric("constant") })
+setGeneric("affine", function(object, expr) { standardGeneric("affine") })
+setGeneric("extract_quadratic_coeffs", function(object, affine_expr, quad_forms) { standardGeneric("extract_quadratic_coeffs") })
+setGeneric("coeff_quad_form", function(object, expr) { standardGeneric("coeff_quad_form") })
+setGeneric("copy", function(object, args = NULL, id_objects = list()) { standardGeneric("copy") })
+setGeneric("tree_copy", function(object, id_objects = list()) { standardGeneric("tree_copy") })
+setGeneric("op_name", function(object) { standardGeneric("op_name") })
+setGeneric("op_func", function(object) { standardGeneric("op_func") })
+## End of newly added generics
