@@ -69,7 +69,8 @@ SOLVER_DEFAULT_PARAM <- list(
     OSQP = list(max_iter = 10000, eps_abs = 1e-5, eps_rel = 1e-5, eps_prim_inf = 1e-4),
     ECOS = list(maxit = 100, abstol = 1e-8, reltol = 1e-8, feastol = 1e-8),
     ECOS_BB = list(maxit = 1000, abstol = 1e-6, reltol = 1e-3, feastol = 1e-6),
-    ##CVX_OPT = list(max_iters = 100, abstol = 1e-7, reltol = 1e-6, feastol = 1e-7, refinement = 1L, kktsolver = "chol"),
+    ## Until cccp fixes the bug I reported, we set the tolerances as below
+    CVXOPT = list(max_iters = 100, abstol = 1e-6, reltol = 1e-6, feastol = 1e-6, refinement = 1L, kktsolver = "chol"),
     SCS = list(max_iters = 2500, eps = 1e-4, alpha = 1.8, scale = 5.0),
     CPLEX = list(itlim = 10000),
     MOSEK = list(num_iter = 10000),
